/*******************************************************************************
 * Copyright (c) 2010 Attila Kovacs <attila_kovacs@post.harvard.edu>.
 * All rights reserved. 
 * 
 * This file is part of minicrush.
 * 
 *     minicrush is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     minicrush is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with minicrush.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Attila Kovacs <attila_kovacs@post.harvard.edu> - initial API and implementation
 ******************************************************************************/
// Copyright (c) 2007 Attila Kovacs 


import java.io.*;
import java.util.*;

import nom.tam.fits.*;


public class APEXSZScan extends LabocaScan {
	
	public APEXSZScan() {
		instrumentName = "aszca";
	    channelTemplate = new APEXSZPixel(-1);
	}
	
	@Override
	public String getFEBE() { return "BOLOSZ-SZACBE"; }
	
	
	public void getChannelGroups() throws Exception {
		boxChannels = LabocaPixel.getChannelGroups(channel, LabocaPixel.class.getField("box"), LabocaPixel.FLAG_DEAD); 
		waferChannels = LabocaPixel.getChannelGroups(channel, LabocaPixel.class.getField("wafer"), LabocaPixel.FLAG_DEAD); 
		cableChannels = LabocaPixel.getChannelGroups(channel, LabocaPixel.class.getField("cable"), LabocaPixel.FLAG_DEAD); 
		ampChannels = LabocaPixel.getChannelGroups(channel, LabocaPixel.class.getField("amp"), LabocaPixel.FLAG_DEAD); 

		pinOrder(cableChannels);
	}
	
	@Override
	public void readPar(BinaryTableHDU hdu) throws IOException, FitsException, HeaderCardException {
		super.readPar(hdu);
		instrumentGain = 1.0;
	}

	public void getWiring() {
		try { APEXSZPixel.readWiring(MiniCRUSH.home + File.separator + instrumentName + File.separator + "wiring.dat"); }
		catch(IOException e) {
			System.err.println("ERROR! Problem reading wiring data. Exiting.");
			System.exit(1);
		}
		
		LabocaPixel.fillWiringInfo(channel);
		System.err.println("   Loaded pixel wiring data.");		
	}
	
	public void yOrder() {
		for(int g=0; g<regionChannels.length; g++) {
			Vector<LabocaPixel> selection = new Vector<LabocaPixel>();
			for(int k=0; k<regionChannels[g].length; k++) selection.add(channel[regionChannels[g][k]]);
			
			Comparator<LabocaPixel> pinorder = new Comparator<LabocaPixel>() {
				public int compare(LabocaPixel c1, LabocaPixel c2) {
					return Double.compare(c1.position.y, c2.position.y);
				}				
			};
			
			Collections.sort(selection, pinorder);
			
			for(int k=0; k<regionChannels[g].length; k++) regionChannels[g][k] = selection.get(k).index;		
		}
		System.err.println("   Regions rearranged to y-order.");	
	}
	

	/*
	public void readMonitor(String fileName)  throws IOException, FitsException, HeaderCardException {
		Fits fits = new Fits(new File(fileName), fileName.endsWith(".gz"));

		BinaryTableHDU hdu = (BinaryTableHDU) fits.getHDU(1);
		readMonitor(hdu);
	}
	*/
	
	
	public void readMonitor(String fileName)  throws IOException, FitsException, HeaderCardException {
		
	}
	
	public void readMonitor(BinaryTableHDU hdu)  throws IOException, FitsException, HeaderCardException {
	}
	
	
	/*
	public void readMonitor(BinaryTableHDU hdu)  throws IOException, FitsException, HeaderCardException {
		System.err.print("   Reading Monitor Table...");
		Header header = hdu.getHeader();
		hdu.getData().getData();
		
		int iMJD = hdu.findColumn("MJD");
		int iLABEL = hdu.findColumn("MONPOINT");
		int iVALUE = hdu.findColumn("MONVALUE");

		int n = header.getIntValue("NAXIS2");
		for(int i=0; i<n; i++) {
			Object[] row = hdu.getRow(i);
			if(((String) row[iLABEL]).endsWith("USTAGETEMP")) {
				System.err.println(row[iLABEL] + "\t" + ((double[]) row[iVALUE])[0]);
			}
		}

		
	}
	*/
	
}
