/*******************************************************************************
 * Copyright (c) 2010 Attila Kovacs <attila_kovacs@post.harvard.edu>.
 * All rights reserved. 
 * 
 * This file is part of minicrush.
 * 
 *     minicrush is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     minicrush is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with minicrush.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Attila Kovacs <attila_kovacs@post.harvard.edu> - initial API and implementation
 ******************************************************************************/
import java.util.*;


public class MJDRange {
	double from = Double.NEGATIVE_INFINITY;
	double to = Double.POSITIVE_INFINITY;
	
	public MJDRange(String spec) throws IllegalArgumentException {
		parse(spec);
	}
	
	public MJDRange(double from, double to) {
		this.from = from;
		this.to = to;		
	}
	
	public boolean contains(double MJD) {
		return MJD >= from && MJD <= to;		
	}
	
	public void parse(String spec) {
		StringTokenizer tokens = new StringTokenizer(spec, "-: \t\r");
		String token = tokens.nextToken();
		try { from = Double.parseDouble(token); }
		catch(NumberFormatException e) {
			if(token.equals("*")) from = Double.NEGATIVE_INFINITY;
			else throw e;
		}
		to = Double.POSITIVE_INFINITY;
		
		if(!tokens.hasMoreTokens()) return;
		
		try { to = Double.parseDouble(token); }
		catch(NumberFormatException e) {
			if(token.equals("*")) to = Double.POSITIVE_INFINITY;
			else throw e;
		}		
	}
	
	public String toString() {
		String text = "";
		if(Double.isInfinite(from)) text += "*";
		else text += from;
		text += "--";
		
		if(Double.isInfinite(to)) text += "*";
		else text += to;
		
		return text;
	}
	
}
