/*******************************************************************************
 * Copyright (c) 2010 Attila Kovacs <attila_kovacs@post.harvard.edu>.
 * All rights reserved. 
 * 
 * This file is part of minicrush.
 * 
 *     minicrush is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     minicrush is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with minicrush.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Attila Kovacs <attila_kovacs@post.harvard.edu> - initial API and implementation
 ******************************************************************************/
import java.io.*;
import java.text.*;
import java.util.*;

public class PArtemisPixel extends LabocaPixel {
	Vector2D size = new Vector2D(4.7 * Unit.arcsec, 4.7 * Unit.arcsec);
	
	public PArtemisPixel(int backendChannel) {
		super(backendChannel);
		area = size.x * size.y;
		domain = area;
	}
	
	public void calcPosition() {
		int row = backendIndex/16;
		int col = backendIndex%16;
		position = new Vector2D(size.x * col, size.y * row);
	}
		
	public String gainString(DecimalFormat df) {
		String text = 
			df.format(gain) + "  " +
			"\t" + df.format(ampGain) + "  "; 
		return text;
	}
	
	@Override
	public void parseGains(StringTokenizer tokens) {
		gain = Double.parseDouble(tokens.nextToken());
		ampGain = Double.parseDouble(tokens.nextToken());
	}
	
	public String gainHeader() {
		return "gain\t\tGmux";
	}

	public static void readWiring(String fileName) throws IOException {	
		int channels = 256;
	
		ampForChannel = new int[channels];
		pinForChannel = new int[channels];
		
		for(int c=0; c<channels; c++) {
			ampForChannel[c] = c / 16;
			pinForChannel[c] = c % 16;
		}
		
	}	
}
