/*******************************************************************************
 * Copyright (c) 2010 Attila Kovacs <attila_kovacs@post.harvard.edu>.
 * All rights reserved. 
 * 
 * This file is part of minicrush.
 * 
 *     minicrush is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     minicrush is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with minicrush.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Attila Kovacs <attila_kovacs@post.harvard.edu> - initial API and implementation
 ******************************************************************************/
import nom.tam.fits.*;

import java.io.*;
import java.util.*;

public class PArtemisScan extends LabocaScan {

	public PArtemisScan() {
		instrumentName = "p-artemis";
		channelTemplate = new PArtemisPixel(-1);
	}
	
	@Override
	public void validate() {
		if(options.containsKey("pixelsize")) {
			System.err.println("   Setting pixel size.");
			StringTokenizer tokens = new StringTokenizer(option("pixelsize"), " \t,xX");
			Vector2D size = new Vector2D();
			size.x = Double.parseDouble(tokens.nextToken()) * Unit.arcsec;
			size.y = tokens.hasMoreTokens() ? Double.parseDouble(tokens.nextToken()) * Unit.arcsec : size.x;
			for(int c=0; c<channels; c++) ((PArtemisPixel) channel[c]).size = size;
		}
		
		System.err.println("   Calculating default pixel positions.");
		for(int c=0; c<channels; c++) {
			((PArtemisPixel) channel[c]).calcPosition();
		}
		LabocaPixel.setReference(channel, channel[referenceChannel]);
		
		if(options.containsKey("rotation")) {
			System.err.println("   Setting array rotation.");
			double angle = Double.parseDouble(option("rotation")) * Unit.deg;
			for(int c=0; c<channels; c++) ((PArtemisPixel) channel[c]).position.rotate(angle);	
		}
			
		System.err.println("   Setting left Nasmyth mount.");
		for(int t=0; t<frames; t++) if(frame[t] != null) {
			frame[t].sinA = -frame[t].horizontal.sinLat;
			frame[t].cosA = frame[t].horizontal.cosLat;
		}
		
		super.validate();
	}
	
	@Override
	public void getWiring() {	
		try { PArtemisPixel.readWiring(MiniCRUSH.home + File.separator + instrumentName + File.separator + "wiring.dat"); }
		catch(IOException e) {
			System.err.println("ERROR! Problem reading wiring data. Exiting.");
			System.exit(1);
		}
		
		SabocaPixel.fillWiringInfo(channel);
		System.err.println("   Loaded pixel wiring data.");
	}
	
	@Override
	public int velocityCut(double minV, double maxV) { return velocityCut(minV, maxV, 1.0 * Unit.s); }
	
	@Override
	public void getChannelGroups() throws Exception {
		ampChannels = LabocaPixel.getChannelGroups(channel, LabocaPixel.class.getField("amp"), LabocaPixel.FLAG_DEAD); 
		pinOrder(ampChannels);
	}
	
	@Override
	public void readPar(BinaryTableHDU hdu) throws IOException, FitsException, HeaderCardException {
		super.readPar(hdu);
		instrumentGain = 1.0;
	}
	
	@Override
	public String getFEBE() { return "ARTEMIS450-ARTBE"; }

	
	public void readMonitor(String fileName)  throws IOException, FitsException, HeaderCardException {
		
	}
	
	public void readMonitor(BinaryTableHDU hdu)  throws IOException, FitsException, HeaderCardException {
	}
	
	
}
