/*******************************************************************************
 * Copyright (c) 2010 Attila Kovacs <attila_kovacs@post.harvard.edu>.
 * All rights reserved. 
 * 
 * This file is part of minicrush.
 * 
 *     minicrush is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     minicrush is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with minicrush.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Attila Kovacs <attila_kovacs@post.harvard.edu> - initial API and implementation
 ******************************************************************************/
import java.io.*;
import java.text.*;
import java.util.*;

public class SabocaPixel extends LabocaPixel {
	
	public SabocaPixel(int backendChannel) {
		super(backendChannel);
		
		double beamFWHM = 7.4 * Unit.arcsec;
		area = 1.14 * beamFWHM * beamFWHM;
		domain = 3.04 * area;
	}

		
	public String gainString(DecimalFormat df) {
		String text = 
			df.format(gain) + "  " +
			"\t" + df.format(ampGain) + "  "; 

		return text;
	}
	
	@Override
	public void parseGains(StringTokenizer tokens) {
		gain = Double.parseDouble(tokens.nextToken());
		ampGain = Double.parseDouble(tokens.nextToken());
	}
	
	public String gainHeader() {
		return "gain\t\tGsquid";
	}

	static Vector2D[] pos; 
	
	public static void readWiring(String fileName) throws IOException {	
		int channels = 40;
		pos = new Vector2D[channels];

		ampForChannel = new int[channels];
		pinForChannel = new int[channels];
				
		BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
		String line;
		while((line = in.readLine()) != null) if(line.length() > 0) if(line.charAt(0) != '#') {
			StringTokenizer tokens = new StringTokenizer(line);
			int c = Integer.parseInt(tokens.nextToken()) - 1;

			ampForChannel[c] = 	Integer.parseInt(tokens.nextToken()) - 1;
			pinForChannel[c] = 	Integer.parseInt(tokens.nextToken()) - 1;
			
			try { pos[c] = new Vector2D(Double.parseDouble(tokens.nextToken()), Double.parseDouble(tokens.nextToken())); }
			catch(Exception e) { pos[c] = new Vector2D(); }
			pos[c].scale(Unit.arcsec);
		}
		
	}	
}
