/*******************************************************************************
 * Copyright (c) 2010 Attila Kovacs <attila_kovacs@post.harvard.edu>.
 * All rights reserved. 
 * 
 * This file is part of minicrush.
 * 
 *     minicrush is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     minicrush is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with minicrush.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Attila Kovacs <attila_kovacs@post.harvard.edu> - initial API and implementation
 ******************************************************************************/
import nom.tam.fits.*;

import java.io.*;

public class SabocaScan extends LabocaScan {

	public SabocaScan() {
		instrumentName = "saboca";
		channelTemplate = new SabocaPixel(-1);
		gainNorm = LabocaPixel.GAINS_BIDIRECTIONAL;
	}
	
	
	public void getWiring() {	
		try { SabocaPixel.readWiring(MiniCRUSH.home + File.separator + instrumentName + File.separator + "wiring.dat"); }
		catch(IOException e) {
			System.err.println("ERROR! Problem reading wiring data. Exiting.");
			System.exit(1);
		}
		
		SabocaPixel.fillWiringInfo(channel);
		System.err.println("   Loaded pixel wiring data.");
	}
	
	public int velocityCut(double minV, double maxV) { return velocityCut(minV, maxV, 1.0 * Unit.s); }
	
	public void getChannelGroups() throws Exception {
		ampChannels = LabocaPixel.getChannelGroups(channel, LabocaPixel.class.getField("amp"), LabocaPixel.FLAG_DEAD); 
		pinOrder(ampChannels);
	}
	
	public void readPar(BinaryTableHDU hdu) throws IOException, FitsException, HeaderCardException {
		super.readPar(hdu);
		instrumentGain = MJD < 55295.0 ? -1000.0 : 1000.0;
	}
	
	@Override
	public String getFEBE() { return "SABOCA-ABBA"; }
	
	
	public void readMonitor(String fileName)  throws IOException, FitsException, HeaderCardException {	
	}
	
	public void readMonitor(BinaryTableHDU hdu)  throws IOException, FitsException, HeaderCardException {
	}
	
}
