/*******************************************************************************
 * Copyright (c) 2010 Attila Kovacs <attila_kovacs@post.harvard.edu>.
 * All rights reserved. 
 * 
 * This file is part of minicrush.
 * 
 *     minicrush is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 * 
 *     minicrush is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with minicrush.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * Contributors:
 *     Attila Kovacs <attila_kovacs@post.harvard.edu> - initial API and implementation
 ******************************************************************************/
// Copyright (c) 2007 Attila Kovacs 

import java.util.*;

class SourceBoxingThread extends Thread {
		public volatile static Vector2D minOffset, maxOffset;
		
		Vector<LabocaScan[]> scans = new Vector<LabocaScan[]>();
		MiniCRUSH crush;
		
		public SourceBoxingThread(MiniCRUSH crush) { this.crush = crush; }
		
		public void addScan(LabocaScan[] scan) { 
			scans.add(scan); 
			for(int k=0; k<scan.length; k++) crush.queue.add(scan[k]);
		}
		
		public void run() {		
			double mindX = Double.POSITIVE_INFINITY, maxdX = Double.NEGATIVE_INFINITY;
			double mindY = Double.POSITIVE_INFINITY, maxdY = Double.NEGATIVE_INFINITY;
			
			final boolean fixSize = crush.options.containsKey("mapsize");
			double fixdX = Double.NaN, fixdY = Double.NaN;
			
			if(fixSize) {
				StringTokenizer sizes = new StringTokenizer(crush.options.get("mapsize"), " \t,:xX");
				fixdX = 0.5* Double.parseDouble(sizes.nextToken()) * Unit.arcsec;
				fixdY = sizes.hasMoreTokens() ? 0.5 * Double.parseDouble(sizes.nextToken()) * Unit.arcsec : fixdX;	
				mindX = -fixdX;
				maxdX = fixdX;
				mindY = -fixdY;
				maxdY = fixdY;
			}
			
			for(LabocaScan[] scan : scans) for(int k=0; k<scan.length; k++) {
				
				final LabocaScan subscan = scan[k];
				final int[] channelSelection = subscan.allSkyChannels;
				final Vector2D offset = new Vector2D();
				
				subscan.map = crush.map;
				
				for(int t=0; t<subscan.frames; t++) {
					final LabocaFrame exposure = subscan.frame[t];
					if(exposure != null) if((exposure.flag & LabocaFrame.SKIP_SOURCE) == 0) {
						boolean valid = false;
						
						for(int q=0; q<channelSelection.length; q++) {
							final int c = channelSelection[q];
							subscan.getMapOffsets(t, c, offset);
							
							if(!fixSize) {
								if(offset.x < mindX) mindX = offset.x;
								else if(offset.x > maxdX) maxdX = offset.x;
								if(offset.y < mindY) mindY = offset.y;
								else if(offset.y > maxdY) maxdY = offset.y;	
							}
							else {
								if(Math.abs(offset.x) > fixdX) exposure.sampleFlag[c] |= LabocaFrame.SAMPLE_SKIP;
								else if(Math.abs(offset.y) > fixdY) exposure.sampleFlag[c] |= LabocaFrame.SAMPLE_SKIP;
								else valid = true;
							}
						}
						
						if(fixSize && ! valid) subscan.frame[t] = null;
					}
				}
				
				updateCorners(mindX, mindY, maxdX, maxdY);
				crush.checkout(subscan);
			}
		}	
		
		public static synchronized void updateCorners(double mindX, double mindY, double maxdX, double maxdY) {
			if(minOffset == null) {
				minOffset = new Vector2D(mindX, mindY);
				maxOffset = new Vector2D(maxdX, maxdY);
			}
			else {
				minOffset.x = Math.min(mindX, minOffset.x);
				minOffset.y = Math.min(mindY, minOffset.y);
				maxOffset.x = Math.max(maxdX, maxOffset.x);
				maxOffset.y = Math.max(maxdY, maxOffset.y);
			}
		}
	}
